unit laskuri;

interface

uses Classes,
{$ifdef CLX}
  QControls, QGraphics,  QStdCtrls,
{$else}  // VCL
  Controls,   Graphics,   StdCtrls,
{$endif}
  SysUtils;

type
  TLaskuri = class;

  TLaskuriEvent = procedure (sender:TLaskuri; diff:integer) of object;

  {
  @abstract(Laskurikomponentti)
  @author(Vesa Lappalainen <vesal@mit.jyu.fi>)
  @created(1996-8-26)
  @lastmod(1998-10-11)


  Tehdn malliksi uusi komponentti TLaskuri, joka on kuten TLabel,
  mutta siin pidetn kokonaislukuarvoa oikeassa reunassa, sinisell pohjalla
  kentn leveys silytten (oletuksena) ja siin on lisksi lisys-metodi.
  Caption-ominaisuuden muuttaminen kelpuuttaa vain kokonaisluvut ja Caption
  on aina synkronissa Count-arvon kanssa.
<br>
  Vesa Lappalainen 26.8.1996
<p>
  Muutokset 15.9.1997/vl
<br>
    + tarkistettu toiminta mys Delphi 3.0:ssa ja kirjoittu vastaava
      komponenttikirjaston "kyttohje"
<p>
  Muutokset 11.10.1998/vl
<br>
    + tarkistettu toiminta mys Delphi 4.0:ssa ja kirjoittu vastaava
      komponenttikirjaston "kyttohje".  Value vaihdettu Count, koska
      luokkatydennin kytt Value:ta lokaalina parametrin
<p>
  "Kirjastoa" voidaan testata ennen varsinaiseen komponenttikirjastoon
  lismist seuraavasti:
  <ol>
    <li> Tee uusi projekti, lis uses-osaan laskuri
    <li> Formin private osaan laita esim:<br>
          @code(lask : TLaskuri)
    <li> Formin alustukseen kirjoitat:<br>
          @code(lask := TLaskuri.Create(self);)<br>
          @code(lask.Parent := self;)<br>
          @code(lask.SetBounds(20,20,50,lask.Height);)<br>
    <li> Lis vaikkapa nappula, jonka painamisen koodiksi kirjoitat:<br>
          @code(lask.Inc(1);)
    </ol>
<p>
  Komponentti voidaan list kirjastoon seuraavasti (Delphi 3.0):
<ol>
    <li> Kopioi laskuri.dcr (tai tee uusi: Tools/Image Editor) samaan paikkaan
       kuin laskuri.pas (tnne on tehty komponenttisivulle
       tuleva bittikartta TLASKURI, 24x24, Huom!  nimi isolla
       Pit olla nkjn oma .dcr eri Delphi versioille!).
    <li> Valitse Component/Install Component
    <li> a) Jos haluamasi kirjasto on jo olemassa, valitse "Into Existing package"
          ja sitten Yes
          <br>
       b) Jos kirjastoa ei ole, valitse "Into New Package", kirjoita
          kirjaston nimi (esim. lask) ja kuvaus ja OK
    <li> Jos kirjastoa (Package) pit korjailla, valitse esim.
       Component/Install Packages
       ja valitse kirjasto ja Edit.  Korjausten jlkeen paina Compile ja
       tarvittaessa Install.  Voit mys avata tiedoston lask.dpk.
</ol>
<p>
  Kirjoittamisvinkki Delphi 4.0:
<ol>
    <li> Lis ensin rivi:  property Count : integer;
    <li> Paina Shift+Ctrl+C  (Complete Class at Cursor)
    <li> Tydenn syntynyt SetCount -metodi.
    <li> Kirjoita rivi: constructor Create(...
    <li> Paina Shift+Ctrl+C
    <li> Tydenn Create-metodi
    <li> Tydenn puuttuvat virtual -mritykset
</ol>
<p>
  Komponentti voidaan list kirjastoon seuraavasti (Delphi 1.0 ja 2.0):
<ol>
    <li> Knn tm tiedosto (vaikka jonkin projekti osana), esim. edellisen
       testin knns riitt
    <li> Kopioi laskuri.dcu sinne kirjastoon, jossa sinulla on komponentit
       (ks. vaikka Components/Install/Search Path)
    <li> Kopioi laskuri.dcr samaan paikkaan (tnne on tehty komponenttisivulle
       tuleva bittikartta TLASKURI, 24x24, Huom!  nimi isolla
       Pit olla nkjn oma .dcr Delphi 1.0:lle ja Delphi 2.0:lle!).
    <li> Valitse Components/Install/Add ja kirjoita Laskuri ja OK OK
       (Delphi 1.0:ssa Options/Install Components/....)
    <li> Jos jatkossa havaitset komponentissa, riitt korjausten jlkeen
       Rebuild Library
</ol>

<p>
  Tehtvi (komponentit voi list thn samaan tiedostoon):
<ol>
    <li> Tee uusi komponentti TLisaaButton, jolla on ominaisuutena
       Laskuri : TLaskuri  - laskuri, jonka arvoa listn aina
                             kun nappia painetaan<br>
       Lisays  : Integer   - paljollako laskuria listn
    <li> Tee autolaskuri em. komponenteilla niin, ett tarvitsee
       kirjoittaa vain yksi rivi koodia: Nollaa-nappulalle
    <li> Tee viel komponentti TNollaa, joka nollaa kaikki samalla
       lomakkeella olevat laskurit (ks. ComponentCount ja Components).
    <li> Tee autolaskuri kirjoittamatta rivikn koodia.  Jos
       listn uusi laskuri-lis -pari, niin taritaanko vielkn
       koodia?
    <li>*Modifioi TNollaa- komponettia siten, ett sille voidaan antaa
       ominaisuutena kaikki komponentit, jotka halutaan nollata.
    <li> Peri TLaskuri panelista ja aseta oletuksena sopivat reunukset plle.
</ol>
}
  TLaskuri = class(TLabel)
  private
    FCount : integer;
    FOnChange: TLaskuriEvent;
    function GetCaption: string;
    procedure SetCaption(const Value: string);
  protected
    procedure SetCount(const Value: integer);  virtual;
  public
    constructor Create(AOwner:TComponent); override;
    function Inc(i:integer) : integer;     virtual;
  published  { Published declarations } { Yleens vain ominaisuudet.           }
    { Ominaisuuksiin ilmoitetaan miten niit luetaan ja miten asetetetaan sek }
    { mahdollinen oletusarvo. Oletusarvo on ainoastaan ohje siit, ettei       }
    { arvoa talleteta resurssitiedostoon, jos arvo on sama kuin oletus.        }
    { Luku/kirjoitusohje voi olla saman tyyppisen attribuutin nimi tai         }
    { aliohjelma asettamiseen ja funktio lukemiseen                            }
    property Count:integer read FCount write SetCount default 0;
    property Caption:string read GetCaption write SetCaption stored false;
    { Seuraavat ominaisuudet islt, ainoastaan oletusarvot erilaiset          }
    property Alignment        default taRightJustify;
    property AutoSize         default False;
    property Color            default clAqua;
    property ParentColor      default false;
    { Kaikki muut isn ominaisuudet tulee sellaisenaan                         }
    property OnChange : TLaskuriEvent read FOnChange write FOnChange;
  end;

procedure Register;

implementation



constructor TLaskuri.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);            { Muista tm!!! Muuten ky huonosti!!!}
  Count := 0;                          { Defaulteissa luvatut arvot:          }
  Alignment := taRightJustify;
  Color := clAqua;
  ParentColor := False;
  Font.Height := -32;
  Font.Style := [fsBold];
  AutoSize := False;
end;


procedure TLaskuri.SetCount(const Value: integer);
var diff : integer;
begin
  diff := Value - Count;
  FCount := Value;
  inherited Caption := IntToStr(Count)+' '; { Vlilynti lop. on paremman nk. }
  if ( Assigned(OnChange) ) then OnChange(self,diff);
end;


function TLaskuri.Inc(i:integer) : integer;
begin
  Count := Count + i;
  Result := Count;
end;




{ Komponentit pit rekisterid komponenttisivulle }
procedure Register;
begin
  RegisterComponents('GKOCLX', [TLaskuri]);
end;

function TLaskuri.GetCaption: string;
begin
  Result := Trim(inherited Caption); { Varo rekursiota!!! }
end;

procedure TLaskuri.SetCaption(const Value: string);
begin
  Count := StrToIntDef(Trim(Value),0);
end;


end.
